import sys

from PyQt5.QtCore import QUrl, pyqtSignal
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEngineProfile
from PyQt5.QtWidgets import QApplication, QDialog, QVBoxLayout


class MyWebEngineView(QWebEngineView):
    def __init__(self, user, pwd, *args, **kwargs):
        super(MyWebEngineView, self).__init__(*args, **kwargs)
        self.cookies = {}
        self._user = user
        self._pwd = pwd
        # 绑定cookie被添加的信号槽
        QWebEngineProfile.defaultProfile().cookieStore().cookieAdded.connect(self.onCookieAdd)
        self.loadFinished.connect(self._on_load_finished)
        self.urlChanged.connect(self._on_load_finished)

    def _on_load_finished(self):
        self.page().toHtml(self.Callable)

    def Callable(self, html_str):
        try:
            self.html = html_str
            js = """var l_name=document.getElementsByName('username');
                    if (l_name.length > 0) {{
                        l_name[0].value = '{}';
                    }};
                    var l_pwd=document.getElementsByName('password');
                    if (l_pwd.length > 0) {{
                        l_pwd[0].value = '{}';
                    }};""".format(self._user, self._pwd)
            self.page().runJavaScript(js)
        except:
            pass
        # except Exception as e:
        #     print("Err:", e)

    def onCookieAdd(self, cookie):
        name = cookie.name().data().decode('utf-8')
        value = cookie.value().data().decode('utf-8')
        self.cookies[name] = value

    def get_cookie(self):
        cookie_dict = {}
        for key, value in self.cookies.items():
            if key in ('ylogin', 'phpdisk_info'):
                cookie_dict[key] = value
        return cookie_dict


class LoginWindow(QDialog):
    cookie = pyqtSignal(object)

    def __init__(self, user=None, pwd=None, gui=False):
        super().__init__()
        self._user = user
        self._pwd = pwd
        self._base_url = 'https://pc.woozooo.com/'
        self._gui = gui
        self.setup()

    def setup(self):
        self.setWindowTitle('滑动滑块，完成登录')
        url = self._base_url + 'account.php?action=login&ref=/mydisk.php'
        QWebEngineProfile.defaultProfile().cookieStore().deleteAllCookies()
        self.web = MyWebEngineView(self._user, self._pwd)
        self.web.urlChanged.connect(self.get_cookie)
        self.web.resize(480, 400)
        self.web.load(QUrl(url))
        self.box = QVBoxLayout(self)
        self.box.addWidget(self.web)

    def get_cookie(self):
        home_url = self._base_url + 'mydisk.php'
        if self.web.url().toString() == home_url:
            cookie = self.web.get_cookie()
            if cookie:
                if self._gui:
                    try:
                        print(";".join([f'{k}={v}' for k, v in cookie.items()]), end='')
                    except:
                        pass
                else:
                    self.cookie.emit(cookie)
                self.reject()


if __name__ == "__main__":
    if len(sys.argv) == 3:
        username, password = sys.argv[1], sys.argv[2]
        app = QApplication(sys.argv)
        form = LoginWindow(username, password, gui=True)
        form.show()
        sys.exit(app.exec())



from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.debug import logger


class ListRefresher(QThread):
    """跟新目录文件与文件夹列表线程"""
    infos = pyqtSignal(object)
    err_msg = pyqtSignal(str, int)

    def __init__(self, parent=None):
        super(ListRefresher, self).__init__(parent)
        self._disk = None
        self._fid = -1
        self.r_files = True
        self.r_folders = True
        self.r_path = True
        self._mutex = QMutex()
        self._is_work = False

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, fid, r_files=True, r_folders=True, r_path=True):
        if not self._is_work:
            self._fid = fid
            self.r_files = r_files
            self.r_folders = r_folders
            self.r_path = r_path
            self.start()
        else:
            self.err_msg.emit("正在更新目录，请稍后再试！", 3100)

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def goto_root_dir(self):
        self._fid = -1
        self.run()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            emit_infos = {}
            # 传递更新内容
            emit_infos['r'] = {'fid': self._fid, 'files': self.r_files, 'folders': self.r_folders, 'path': self.r_path}
            try:
                if self.r_files:
                    # [i.id, i.name, i.size, i.time, i.downs, i.has_pwd, i.has_des]
                    info = {i.id: i for i in self._disk.get_file_list(self._fid)}

                    emit_infos['file_list'] = {key: info.get(key) for key in sorted(info.keys(), reverse=True)}

                if self.r_folders:
                    folders, full_path = self._disk.get_dir_list(self._fid)
                    if not full_path and not folders and self._fid != -1:
                        self.err_msg.emit(f"文件夹id {self._fid} 不存在，将切换到更目录", 2900)
                        self._is_work = False
                        self._mutex.unlock()
                        return self.goto_root_dir()
                    info = {i.name: i for i in folders}
                    emit_infos['folder_list'] = {key: info.get(key) for key in sorted(info.keys())}
                    emit_infos['path_list'] = full_path
            except TimeoutError:
                self.err_msg.emit("网络超时，无法更新目录，稍后再试！", 7000)
            except Exception as e:
                self.err_msg.emit("未知错误，无法更新目录，稍后再试！", 7000)
                logger.error(f"ListRefresher error: e={e}")
            else:
                self.infos.emit(emit_infos)
            self._is_work = False
            self._mutex.unlock()



import os
from pickle import load

from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QPixmap, QIcon
from PyQt6.QtWidgets import (
    QDialog,
    QLabel,
    QDialogButtonBox,
    QVBoxLayout,
    QHBoxLayout,
    QFileDialog,
)

from lanzou.debug import SRC_DIR
from lanzou.gui.others import MyLineEdit
from lanzou.gui.qss import others_style, btn_style


def get_minimum_file(file_lst: list) -> str:
    """ 返回大小最小的文件名 """
    if not file_lst:
        return ""
    if len(file_lst) == 1:
        return file_lst[0]
    res = file_lst[0]
    size = os.path.getsize(res)
    for f in file_lst[1:]:
        _size = os.path.getsize(f)
        if _size < size:
            res = f
            size = _size

    return res


def un_serialize(folder):
    """反序列化文件信息数据"""
    txt_lst = []
    folder_lst = os.listdir(folder)
    for item in folder_lst:
        _file = os.path.join(folder, item)
        if os.path.isfile(_file):
            if _file.endswith(".txt"):
                txt_lst.append(_file)
    record_file = get_minimum_file(txt_lst)
    if not record_file:
        msg = "没有 txt 记录文件"
        return False, msg
    record = {}
    try:
        with open(record_file, "rb") as f_handle:
            _record = load(f_handle)
        if isinstance(_record, dict):
            record = _record
    except Exception as e:  # 这里可能会丢奇怪的异常
        # logger.debug(f"Pickle e={e}")
        pass
    if not record:
        msg = f"{record_file} : 记录文件不对"
        return False, msg
    else:
        files_info = {}
        not_complete = False
        for _file in record["parts"]:
            full_path = os.path.join(folder, _file)
            if not os.path.isfile(full_path):
                not_complete = True
                files_info[_file] = False

        if not_complete:
            msg = "文件不全"
            return False, msg
        merged_file_name = os.path.join(folder, record["name"])
        with open(merged_file_name, "ab") as merge_f:
            for _file in record["parts"]:
                part_file_name = os.path.join(folder, _file)
                with open(part_file_name, "rb") as f:
                    for data in f:
                        merge_f.write(data)

        if os.path.getsize(merged_file_name) == record["size"]:
            for _file in record["parts"]:
                part_file_name = os.path.join(folder, _file)
                os.remove(part_file_name)
            os.remove(record_file)
            return True, ""
        else:
            msg = "文件大小对不上"

        return False, msg


class MergeFileDialog(QDialog):
    check_update = pyqtSignal(str, bool)

    def __init__(self, user_home, parent=None):
        super(MergeFileDialog, self).__init__(parent)
        self.cwd = user_home
        self.selected = ""
        self.initUI()
        self.setStyleSheet(others_style)

    def initUI(self):
        self.setWindowTitle("合并文件")
        self.setWindowIcon(QIcon(SRC_DIR + "upload.png"))
        self.logo = QLabel()
        self.logo.setPixmap(QPixmap(SRC_DIR + "logo3.gif"))
        self.logo.setStyleSheet("background-color:rgb(0,153,255);")
        self.logo.setAlignment(Qt.AlignmentFlag.AlignCenter)

        # lable
        self.choose_lb = QLabel("选择文件夹")
        # folder
        self.choose_folder = MyLineEdit(self)
        self.choose_folder.setObjectName("choose_folder")
        self.choose_folder.clicked.connect(self.slot_choose_folder)
        self.status = QLabel(self)

        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setText("提取")
        self.buttonBox.button(QDialogButtonBox.StandardButton.Cancel).setText("关闭")
        self.buttonBox.setStyleSheet(btn_style)

        vbox = QVBoxLayout()
        hbox_head = QHBoxLayout()
        hbox_button = QHBoxLayout()
        hbox_head.addWidget(self.choose_lb)
        hbox_head.addWidget(self.choose_folder)
        hbox_button.addWidget(self.buttonBox)
        vbox.addWidget(self.logo)
        vbox.addStretch(1)
        vbox.addWidget(self.status)
        vbox.addLayout(hbox_head)
        vbox.addStretch(1)
        vbox.addLayout(hbox_button)
        self.setLayout(vbox)
        self.setMinimumWidth(350)

        # 设置信号
        self.buttonBox.accepted.connect(self.slot_btn_ok)
        self.buttonBox.rejected.connect(self.slot_btn_no)
        self.buttonBox.rejected.connect(self.reject)

    def slot_choose_folder(self):
        dir_choose = QFileDialog.getExistingDirectory(self, "选择文件夹", self.cwd)  # 起始路径
        if dir_choose == "":
            return
        self.selected = dir_choose
        self.choose_folder.setText(self.selected)
        self.status.setText("")
        self.cwd = os.path.dirname(dir_choose)

    def slot_btn_no(self):
        self.selected = ""
        self.choose_folder.setText(self.selected)
        self.status.setText("")

    def slot_btn_ok(self):
        if self.selected:
            success, msg = un_serialize(self.selected)
            if success:
                text = "提取成功✅"
            else:
                text = f"提取失败❌, {msg}"
        else:
            text = "未选择文件夹📂"
        self.status.setText(text)



from lanzou.gui.dialogs.about import AboutDialog
from lanzou.gui.dialogs.captcha import CaptchaDialog
from lanzou.gui.dialogs.delete import DeleteDialog
from lanzou.gui.dialogs.info import InfoDialog
from lanzou.gui.dialogs.login import LoginDialog
from lanzou.gui.dialogs.merge_file import MergeFileDialog
from lanzou.gui.dialogs.move import MoveFileDialog
from lanzou.gui.dialogs.rec_folder import RecFolderDialog
from lanzou.gui.dialogs.rename import RenameDialog
from lanzou.gui.dialogs.setpwd import SetPwdDialog
from lanzou.gui.dialogs.setting import SettingDialog
from lanzou.gui.dialogs.upload import UploadDialog

__ALL__ = ['LoginDialog', 'UploadDialog', 'InfoDialog', 'RenameDialog',
           'SettingDialog', 'RecFolderDialog', 'SetPwdDialog', 'MoveFileDialog',
           'DeleteDialog', 'KEY', 'AboutDialog', 'CaptchaDialog', 'MergeFileDialog']



import re

from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.api import LanZouCloud
from lanzou.api.utils import is_folder_url, is_file_url
from lanzou.debug import logger


class GetSharedInfo(QThread):
    '''提取界面获取分享链接信息'''
    infos = pyqtSignal(object)
    msg = pyqtSignal(str, int)
    update = pyqtSignal()
    clean = pyqtSignal()

    def __init__(self, parent=None):
        super(GetSharedInfo, self).__init__(parent)
        self._disk = None
        self.share_url = ""
        self.pwd = ""
        self.is_file = ""
        self.is_folder = ""
        self._mutex = QMutex()
        self._is_work = False
        self._pat = r"(https?://(\w[-\w]*\.)?lanzou[a-z].com/[a-z]?[-/a-zA-Z0-9]+)[^a-zA-Z0-9]*([a-zA-Z0-9]+\w+)?"

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, text, pwd_input=""):
        '''获取分享链接信息'''
        text = text.strip()
        pwd_input = pwd_input.strip()
        if not text:
            self.update.emit()
            return None
        for share_url, _, pwd in re.findall(self._pat, text):
            if is_file_url(share_url):  # 文件链接
                is_file = True
                is_folder = False
                self.msg.emit("正在获取文件链接信息……", 20000)
            elif is_folder_url(share_url):  # 文件夹链接
                is_folder = True
                is_file = False
                self.msg.emit("正在获取文件夹链接信息，可能需要几秒钟，请稍候……", 500000)
            else:
                self.msg.emit(f"{share_url} 为非法链接！", 0)
                self.update.emit()
                return None
            self.clean.emit()  # 清理旧的显示信息
            self.share_url = share_url
            if pwd_input:
                self.pwd = pwd_input
            elif pwd:
                self.pwd = pwd
            else:  # 一个或两个汉字的提取码
                pwd_ = text.split(' ')[-1].split('：')[-1].split(':')[-1]
                self.pwd = pwd_ if 1 <= len(pwd_) <= 2 else ''
            self.is_file = is_file
            self.is_folder = is_folder
            self.start()
            break

    def __del__(self):
        self.wait()

    def stop(self):  # 用于手动停止
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def emit_msg(self, infos):
        '''根据查询信息发送状态信号'''
        show_time = 2999  # 提示显示时间，单位 ms
        if infos.code == LanZouCloud.FILE_CANCELLED:
            self.msg.emit("<font color='red'>文件不存在，或已删除！</font>", show_time)
        elif infos.code == LanZouCloud.URL_INVALID:
            self.msg.emit("<font color='red'>链接非法！</font>", show_time)
        elif infos.code == LanZouCloud.PASSWORD_ERROR:
            self.msg.emit(f"<font color='red'>提取码 [<b><font color='magenta'>{self.pwd}</font></b>] 错误！</font>",
                          show_time)
        elif infos.code == LanZouCloud.LACK_PASSWORD:
            self.msg.emit("<font color='red'>请在链接后面跟上提取码，空格分割！</font>", show_time)
        elif infos.code == LanZouCloud.NETWORK_ERROR:
            self.msg.emit("<font color='red'>网络错误！</font>", show_time)
        elif infos.code == LanZouCloud.SUCCESS:
            self.msg.emit("<font color='#00CC00'>提取成功！</font>", show_time)
        else:
            self.msg.emit(f"<font color='red'>未知错误 code={infos.code}！</font>", show_time * 4)

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            try:
                if self.is_file:  # 链接为文件
                    _infos = self._disk.get_share_info_by_url(self.share_url, self.pwd)
                    self.emit_msg(_infos)
                    self.infos.emit(_infos)
                elif self.is_folder:  # 链接为文件夹
                    _infos = self._disk.get_folder_info_by_url(self.share_url, self.pwd)
                    self.emit_msg(_infos)
                    self.infos.emit(_infos)
                else:
                    logger.error(f"GetShareInfo error: Not a file or folder!")
            except TimeoutError:
                self.msg.emit("font color='red'>网络超时！请稍后重试</font>", 5000)
            except Exception as e:
                self.msg.emit(f"font color='red'>未知错误：{e}</font>", 5000)
                logger.error(f"GetShareInfo error: e={e}")
            self._is_work = False
            self.update.emit()
            self._mutex.unlock()
        else:
            self.msg.emit("<font color='blue'>后台正在运行，稍后重试！</font>", 4000)



import os

from PyQt6.QtCore import QThread, pyqtSignal

from lanzou.api import LanZouCloud
from lanzou.debug import logger


class Uploader(QThread):
    """单个文件上传线程"""
    finished_ = pyqtSignal(object)
    folder_file_failed = pyqtSignal(object, object)
    failed = pyqtSignal()
    proc = pyqtSignal(str)
    update = pyqtSignal()

    def __init__(self, disk, task, callback, allow_big_file=False, parent=None):
        super(Uploader, self).__init__(parent)
        self._disk = disk
        self._task = task
        self._allow_big_file = allow_big_file
        self._callback_thread = callback(task)

    def stop(self):
        self.terminate()

    def _callback(self):
        """显示进度条的回调函数"""
        if not self._callback_thread.isRunning():
            self.update.emit()
            self.proc.emit(self._callback_thread.emit_msg)
            self._callback_thread.start()

    def _down_failed(self, code, file):
        """显示下载失败的回调函数"""
        self.folder_file_failed.emit(code, file)

    def __del__(self):
        self.wait()

    def run(self):
        if not self._task:
            logger.error("Upload task is empty!")
            return None
        self._task.run = True
        try:
            if os.path.isdir(self._task.url):
                code, fid, isfile = self._disk.upload_dir(self._task, self._callback, self._allow_big_file)
            else:
                code, fid, isfile = self._disk.upload_file(self._task, self._task.url, self._task.fid,
                                                           self._callback, self._allow_big_file)
        except TimeoutError:
            self._task.info = LanZouCloud.NETWORK_ERROR
            self.update.emit()
        except Exception as err:
            logger.error(f"Upload error: err={err}")
            self._task.info = err
            self.update.emit()
        else:
            if code == LanZouCloud.SUCCESS:
                if self._task.pwd:
                    self._disk.set_passwd(fid, self._task.pwd, is_file=isfile)
                if self._task.desc:
                    self._disk.set_desc(fid, self._task.desc, is_file=isfile)
                self._task.rate = 1000  # 回调线程可能在休眠
            else:
                self.failed.emit()
        self._task.run = False
        self.finished_.emit(self._task)



from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QIcon, QPixmap
from PyQt6.QtWidgets import (QLineEdit, QDialog, QLabel, QFormLayout,
                             QDialogButtonBox, QVBoxLayout)

from lanzou.debug import SRC_DIR
from lanzou.gui.others import AutoResizingTextEdit
from lanzou.gui.qss import dialog_qss_style


class InfoDialog(QDialog):
    """文件信息对话框"""
    get_dl_link = pyqtSignal(str, str)
    closed = pyqtSignal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.infos = None
        self._short_link_flag = True  # 防止多次重试
        self.initUI()
        self.setStyleSheet(dialog_qss_style)

    def update_ui(self):
        self.tx_dl_link.setPlaceholderText("单击获取")
        self.tx_name.setText(self.infos.name)
        if self.infos.is_file:
            self.setWindowTitle("文件信息")
            self.lb_name.setText("文件名：")
            self.lb_desc.setText("文件描述：")
            self.tx_dl_link.setText("")  # 清空旧的信息
            self.tx_short.setText("")  # 清空旧的信息
            self.lb_dl_link.setVisible(True)
            self.tx_dl_link.setVisible(True)
        else:
            self.setWindowTitle("文件夹信息")
            self.lb_name.setText("文件夹名：")
            self.lb_desc.setText("文件夹描述：")
            self.lb_dl_link.setVisible(False)
            self.tx_dl_link.setVisible(False)

        if self.infos.size:
            self.tx_size.setText(self.infos.size)
            self.lb_size.setVisible(True)
            self.tx_size.setVisible(True)
        else:
            self.tx_size.setVisible(False)
            self.lb_size.setVisible(False)

        if self.infos.time:
            self.lb_time.setVisible(True)
            self.tx_time.setVisible(True)
            self.tx_time.setText(self.infos.time)
        else:
            self.lb_time.setVisible(False)
            self.tx_time.setVisible(False)

        if self.infos.downs:
            self.lb_dl_count.setVisible(True)
            self.tx_dl_count.setVisible(True)
            self.tx_dl_count.setText(str(self.infos.downs))
        else:
            self.tx_dl_count.setVisible(False)
            self.lb_dl_count.setVisible(False)

        if self.infos.pwd:
            self.tx_pwd.setText(self.infos.pwd)
            self.tx_pwd.setPlaceholderText("")
        else:
            self.tx_pwd.setText("")
            self.tx_pwd.setPlaceholderText("无")

        if self.infos.desc:
            self.tx_desc.setText(self.infos.desc)
            self.tx_desc.setPlaceholderText("")
        else:
            self.tx_desc.setText("")
            self.tx_desc.setPlaceholderText("无")

        self.tx_share_url.setText(self.infos.url)
        self.adjustSize()

    def set_values(self, infos):
        self.infos = infos
        self.update_ui()

    def set_dl_link_tx(self, text):
        self.tx_dl_link.setText(text)
        self.adjustSize()

    def call_get_dl_link(self):
        url = self.tx_share_url.text()
        pwd = self.tx_pwd.text()
        self.get_dl_link.emit(url, pwd)
        self.tx_dl_link.setPlaceholderText("后台获取中，请稍候！")

    def call_get_short_url(self):
        if self._short_link_flag:
            self._short_link_flag = False
            self.tx_short.setPlaceholderText("后台获取中，请稍候！")
            url = self.tx_dl_link.toPlainText()
            if not url:
                url = self.tx_share_url.text()
            print(url)
            from lanzou.api.extra import get_short_url
            short_url = get_short_url(url)
            if short_url:
                self.tx_short.setText(short_url)
                self.tx_short.setPlaceholderText("单击获取")
                self._short_link_flag = True
            else:
                self.tx_short.setText("")
                self.tx_short.setPlaceholderText("生成失败！api 可能已经失效")

    def clean(self):
        self._short_link_flag = True
        self.tx_short.setText("")
        self.tx_short.setPlaceholderText("单击获取")

    def initUI(self):
        self.setWindowIcon(QIcon(SRC_DIR + "share.png"))
        self.setWindowTitle("文件信息")
        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Close)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Close).setText("关闭")
        self.buttonBox.rejected.connect(self.reject)
        self.buttonBox.rejected.connect(self.clean)
        self.buttonBox.rejected.connect(self.closed.emit)

        self.logo = QLabel()
        self.logo.setPixmap(QPixmap(SRC_DIR + "q9.gif"))
        self.logo.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.logo.setStyleSheet("background-color:rgb(255,204,51);")

        self.lb_name = QLabel()
        self.lb_name.setText("文件名：")
        self.tx_name = AutoResizingTextEdit()
        self.tx_name.setReadOnly(True)
        self.tx_name.setMinimumLines(1)

        self.lb_size = QLabel()
        self.lb_size.setText("文件大小：")
        self.tx_size = QLabel()

        self.lb_time = QLabel()
        self.lb_time.setText("上传时间：")
        self.tx_time = QLabel()

        self.lb_dl_count = QLabel()
        self.lb_dl_count.setText("下载次数：")
        self.tx_dl_count = QLabel()

        self.lb_share_url = QLabel()
        self.lb_share_url.setText("分享链接：")
        self.tx_share_url = QLineEdit()
        self.tx_share_url.setReadOnly(True)

        self.lb_pwd = QLabel()
        self.lb_pwd.setText("提取码：")
        self.tx_pwd = QLineEdit()
        self.tx_pwd.setReadOnly(True)

        self.lb_short = QLabel()
        self.lb_short.setText("短链接：")
        self.tx_short = AutoResizingTextEdit(self)
        self.tx_short.setPlaceholderText("单击获取")
        self.tx_short.clicked.connect(self.call_get_short_url)
        self.tx_short.setReadOnly(True)
        self.tx_short.setMinimumLines(1)

        self.lb_desc = QLabel()
        self.lb_desc.setText("文件描述：")
        self.tx_desc = AutoResizingTextEdit()
        self.tx_desc.setReadOnly(True)
        self.tx_desc.setMinimumLines(1)

        self.lb_dl_link = QLabel()
        self.lb_dl_link.setText("下载直链：")
        self.tx_dl_link = AutoResizingTextEdit(self)
        self.tx_dl_link.setPlaceholderText("单击获取")
        self.tx_dl_link.clicked.connect(self.call_get_dl_link)
        self.tx_dl_link.setReadOnly(True)
        self.tx_dl_link.setMinimumLines(1)

        vbox = QVBoxLayout()
        vbox.addWidget(self.logo)
        vbox.addStretch(1)
        form = QFormLayout()
        form.setLabelAlignment(Qt.AlignmentFlag.AlignRight)
        form.addRow(self.lb_name, self.tx_name)
        form.addRow(self.lb_size, self.tx_size)
        form.addRow(self.lb_time, self.tx_time)
        form.addRow(self.lb_dl_count, self.tx_dl_count)
        form.addRow(self.lb_share_url, self.tx_share_url)
        form.addRow(self.lb_pwd, self.tx_pwd)
        form.addRow(self.lb_short, self.tx_short)
        form.addRow(self.lb_desc, self.tx_desc)
        form.addRow(self.lb_dl_link, self.tx_dl_link)
        form.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)  # 覆盖MacOS的默认样式
        vbox.addLayout(form)
        vbox.addStretch(1)
        vbox.addWidget(self.buttonBox)

        self.setLayout(vbox)
        self.setMinimumWidth(500)



"""
API 处理后返回的数据类型
"""

from collections import namedtuple

from lanzou.api.models import FolderList

File = namedtuple('File', ['name', 'id', 'time', 'size', 'type', 'downs', 'has_pwd', 'has_des'])
Folder = namedtuple('Folder', ['name', 'id', 'has_pwd', 'desc'])
FolderId = namedtuple('FolderId', ['name', 'id', 'desc', 'now'])
RecFile = namedtuple('RecFile', ['name', 'id', 'type', 'size', 'time'])
RecFolder = namedtuple('RecFolder', ['name', 'id', 'size', 'time', 'files'])
FileDetail = namedtuple('FileDetail', ['code', 'name', 'size', 'type', 'time', 'desc', 'pwd', 'url', 'durl'],
                        defaults=(0, *[''] * 8))
ShareInfo = namedtuple('ShareInfo', ['code', 'name', 'url', 'pwd', 'desc', 'time', 'size'], defaults=(0, *[''] * 6))
DirectUrlInfo = namedtuple('DirectUrlInfo', ['code', 'name', 'durl'])
FolderInfo = namedtuple('Folder', ['name', 'id', 'pwd', 'time', 'desc', 'url', 'size', 'size_int', 'count'],
                        defaults=(*('',) * 7, 0, 0))
FileInFolder = namedtuple('FileInFolder', ['name', 'time', 'size', 'type', 'url', 'pwd'], defaults=('',) * 6)
FolderDetail = namedtuple('FolderDetail', ['code', 'folder', 'files', 'sub_folders'],
                          defaults=(0, FolderInfo(), FileInFolder(), FolderList()))

# gui 提取界面 name.setData
ShareItem = namedtuple('ShareItem', ['item', 'all', 'count', 'parent'], defaults=('', None, 1, []))



from pickle import load, dump

from lanzou.debug import CONFIG_FILE, DL_DIR

__all__ = ['config']

KEY = 152  # config 加密 key

default_settings = {
    "download_threads": 3,  # 同时三个下载任务
    "timeout": 5,  # 每个请求的超时 s(不包含下载响应体的用时)
    "max_size": 100,  # 单个文件大小上限 MB
    "dl_path": DL_DIR,
    "time_fmt": False,  # 是否使用年月日时间格式
    "to_tray": False,  # 关闭到系统托盘
    "watch_clipboard": False,  # 监听系统剪切板
    "debug": False,  # 调试
    "set_pwd": False,
    "pwd": "",
    "set_desc": False,
    "desc": "",
    "upload_delay": 20,  # 上传大文件延时 0 - 20s
    "allow_big_file": False,
    "upgrade": True
}


def encrypt(key, s):
    b = bytearray(str(s).encode("utf-8"))
    n = len(b)
    c = bytearray(n * 2)
    j = 0
    for i in range(0, n):
        b1 = b[i]
        b2 = b1 ^ key
        c1 = b2 % 19
        c2 = b2 // 19
        c1 = c1 + 46
        c2 = c2 + 46
        c[j] = c1
        c[j + 1] = c2
        j = j + 2
    return c.decode("utf-8")


def decrypt(ksa, s):
    c = bytearray(str(s).encode("utf-8"))
    n = len(c)
    if n % 2 != 0:
        return ""
    n = n // 2
    b = bytearray(n)
    j = 0
    for i in range(0, n):
        c1 = c[j]
        c2 = c[j + 1]
        j = j + 2
        c1 = c1 - 46
        c2 = c2 - 46
        b2 = c2 * 19 + c1
        b1 = b2 ^ ksa
        b[i] = b1
    return b.decode("utf-8")


def save_config(cf):
    with open(CONFIG_FILE, 'wb') as f:
        dump(cf, f)


class Config:
    """存储登录用户信息"""

    def __init__(self):
        self._users = {}
        self._cookie = ''
        self._name = ''
        self._pwd = ''
        self._work_id = -1
        self._settings = default_settings

    @staticmethod
    def encode(var):
        if isinstance(var, dict):
            for k, v in var.items():
                var[k] = encrypt(KEY, str(v))
        elif var:
            var = encrypt(KEY, str(var))
        return var

    @staticmethod
    def decode(var):
        try:
            if isinstance(var, dict):
                dvar = {}  # 新开内存，否则会修改原字典
                for k, v in var.items():
                    dvar[k] = decrypt(KEY, str(v))
            elif var:
                dvar = decrypt(KEY, var)
            else:
                dvar = None
        except Exception:
            dvar = None
        return dvar

    def update_user(self):
        if self._name:
            self._users[self._name] = (self._cookie, self._name, self._pwd,
                                       self._work_id, self._settings)
            save_config(self)

    def del_user(self, name) -> bool:
        name = self.encode(name)
        if name in self._users:
            del self._users[name]
            return True
        return False

    def change_user(self, name) -> bool:
        name = self.encode(name)
        if name in self._users:
            self.update_user()  # 切换用户前保持目前用户信息
            user = self._users[name]
            self._cookie = user[0]
            self._name = user[1]
            self._pwd = user[2]
            self._work_id = user[3]
            self._settings = user[4]
            save_config(self)
            return True
        return False

    @property
    def users_name(self) -> list:
        return [self.decode(user) for user in self._users]

    def get_user_info(self, name):
        """返回用户名、pwd、cookie"""
        en_name = self.encode(name)
        if en_name in self._users:
            user_info = self._users[en_name]
            return name, self.decode(user_info[2]), self.decode(user_info[0])

    def default_path(self):
        path = default_settings['dl_path']
        self._settings.update({'dl_path': path})
        save_config(self)

    @property
    def default_settings(self):
        return default_settings

    @property
    def name(self):
        return self.decode(self._name)

    @property
    def pwd(self):
        return self.decode(self._pwd)

    @property
    def cookie(self):
        return self.decode(self._cookie)

    @cookie.setter
    def cookie(self, cookie):
        self._cookie = self.encode(cookie)
        save_config(self)

    @property
    def work_id(self):
        return self._work_id

    @work_id.setter
    def work_id(self, work_id):
        self._work_id = work_id
        save_config(self)

    def set_cookie(self, cookie):
        self._cookie = self.encode(cookie)
        save_config(self)

    def set_username(self, username):
        self._name = self.encode(username)
        save_config(self)

    @property
    def path(self):
        return self._settings['dl_path']

    @path.setter
    def path(self, path):
        self._settings.update({'dl_path': path})
        save_config(self)

    @property
    def settings(self):
        return self._settings

    @settings.setter
    def settings(self, settings):
        self._settings = settings
        save_config(self)

    def set_infos(self, infos: dict):
        self.update_user()  # 切换用户前保持目前用户信息
        if "name" in infos:
            self._name = self.encode(infos["name"])
        if "pwd" in infos:
            self._pwd = self.encode(infos["pwd"])
        if "cookie" in infos:
            self._cookie = self.encode(infos["cookie"])
        if "path" in infos:
            self._settings.update({'dl_path': infos["path"]})
        if "work_id" in infos:
            self._work_id = infos["work_id"]
        if "settings" in infos:
            self._settings = infos["settings"]
        save_config(self)


# 全局配置对象
try:
    with open(CONFIG_FILE, 'rb') as c:
        config = load(c)
except:
    config = Config()



import os

from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QIcon, QPixmap, QStandardItem, QStandardItemModel
from PyQt6.QtWidgets import (QDialog, QLabel, QDialogButtonBox, QPushButton, QListView,
                             QVBoxLayout, QHBoxLayout, QAbstractItemView, QFileDialog)

from lanzou.debug import SRC_DIR
from lanzou.gui.models import UpJob
from lanzou.gui.others import MyListView
from lanzou.gui.qss import dialog_qss_style


class UploadDialog(QDialog):
    """文件上传对话框"""
    new_infos = pyqtSignal(object)

    def __init__(self, user_home):
        super().__init__()
        self.cwd = user_home
        self._folder_id = -1
        self._folder_name = "LanZouCloud"
        self.set_pwd = False
        self.set_desc = False
        self.pwd = ''
        self.desc = ''
        self.allow_big_file = False
        self.max_size = 100
        self.selected = []
        self.initUI()
        self.set_size()
        self.setStyleSheet(dialog_qss_style)

    def set_pwd_desc_bigfile(self, settings):
        self.set_pwd = settings["set_pwd"]
        self.set_desc = settings["set_desc"]
        self.pwd = settings["pwd"]
        self.desc = settings["desc"]
        self.allow_big_file = settings["allow_big_file"]
        self.max_size = settings["max_size"]
        if self.allow_big_file:
            self.btn_chooseMultiFile.setToolTip("")
        else:
            self.btn_chooseMultiFile.setToolTip(f"文件大小上限 {self.max_size}MB")

    def set_values(self, folder_name, folder_id, files):
        self.setWindowTitle("上传文件至 ➩ " + str(folder_name))
        self._folder_id = folder_id
        self._folder_name = folder_name
        if files:
            self.selected = files
            self.show_selected()
        self.exec()

    def initUI(self):
        self.setWindowTitle("上传文件")
        self.setWindowIcon(QIcon(SRC_DIR + "upload.png"))
        self.logo = QLabel()
        self.logo.setPixmap(QPixmap(SRC_DIR + "logo3.gif"))
        self.logo.setStyleSheet("background-color:rgb(0,153,255);")
        self.logo.setAlignment(Qt.AlignmentFlag.AlignCenter)

        # btn 1
        self.btn_chooseDir = QPushButton("选择文件夹", self)
        self.btn_chooseDir.setObjectName("btn_chooseDir")
        self.btn_chooseDir.setObjectName("btn_chooseDir")
        self.btn_chooseDir.setIcon(QIcon(SRC_DIR + "folder.gif"))

        # btn 2
        self.btn_chooseMultiFile = QPushButton("选择多文件", self)
        self.btn_chooseDir.setObjectName("btn_chooseMultiFile")
        self.btn_chooseMultiFile.setObjectName("btn_chooseMultiFile")
        self.btn_chooseMultiFile.setIcon(QIcon(SRC_DIR + "file.png"))

        # btn 3
        self.btn_deleteSelect = QPushButton("移除", self)
        self.btn_deleteSelect.setObjectName("btn_deleteSelect")
        self.btn_deleteSelect.setIcon(QIcon(SRC_DIR + "delete.png"))
        self.btn_deleteSelect.setToolTip("按 Delete 移除选中文件")

        # 列表
        self.list_view = MyListView()
        self.list_view.drop_files.connect(self.add_drop_files)
        self.list_view.setViewMode(QListView.ViewMode.ListMode)
        self.slm = QStandardItem()
        self.model = QStandardItemModel()
        self.list_view.setModel(self.model)
        self.model.removeRows(0, self.model.rowCount())  # 清除旧的选择
        self.list_view.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        self.list_view.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.list_view.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)

        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setText("确定")
        self.buttonBox.button(QDialogButtonBox.StandardButton.Cancel).setText("取消")

        vbox = QVBoxLayout()
        hbox_head = QHBoxLayout()
        hbox_button = QHBoxLayout()
        hbox_head.addWidget(self.btn_chooseDir)
        hbox_head.addStretch(1)
        hbox_head.addWidget(self.btn_chooseMultiFile)
        hbox_button.addWidget(self.btn_deleteSelect)
        hbox_button.addStretch(1)
        hbox_button.addWidget(self.buttonBox)
        vbox.addWidget(self.logo)
        vbox.addLayout(hbox_head)
        vbox.addWidget(self.list_view)
        vbox.addLayout(hbox_button)
        self.setLayout(vbox)
        self.setMinimumWidth(350)

        # 设置信号
        self.btn_chooseDir.clicked.connect(self.slot_btn_chooseDir)
        self.btn_chooseMultiFile.clicked.connect(self.slot_btn_chooseMultiFile)
        self.btn_deleteSelect.clicked.connect(self.slot_btn_deleteSelect)

        self.buttonBox.accepted.connect(self.slot_btn_ok)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.clear_old)
        self.buttonBox.rejected.connect(self.reject)

    def set_size(self):
        if self.selected:
            h = 18 if len(self.selected) > 18 else 10
            w = 40
            for i in self.selected:
                i_len = len(i)
                if i_len > 100:
                    w = 100
                    break
                if i_len > w:
                    w = i_len
            self.resize(120 + w * 7, h * 30)
        else:
            self.resize(400, 300)

    def clear_old(self):
        self.selected = []
        self.model.removeRows(0, self.model.rowCount())
        self.set_size()

    def show_selected(self):
        self.model.removeRows(0, self.model.rowCount())
        for item in self.selected:
            if os.path.isfile(item):
                self.model.appendRow(QStandardItem(QIcon(SRC_DIR + "file.png"), item))
            else:
                self.model.appendRow(QStandardItem(QIcon(SRC_DIR + "folder.gif"), item))
            self.set_size()

    def backslash(self):
        """Windows backslash"""
        tasks = {}
        for item in self.selected:
            url = os.path.normpath(item)
            total_size = 0
            total_file = 0
            if os.path.isfile(url):
                total_size = os.path.getsize(url)
                if not total_size:
                    continue  # 空文件无法上传
                total_file += 1
            else:
                for filename in os.listdir(url):
                    file_path = os.path.join(url, filename)
                    if not os.path.isfile(file_path):
                        continue  # 跳过子文件夹
                    total_size += os.path.getsize(file_path)
                    total_file += 1
            tasks[url] = UpJob(url=url,
                               fid=self._folder_id,
                               folder=self._folder_name,
                               pwd=self.pwd if self.set_pwd else None,
                               desc=self.desc if self.set_desc else None,
                               total_size=total_size,
                               total_file=total_file)
        return tasks

    def slot_btn_ok(self):
        tasks = self.backslash()
        if self.selected:
            self.new_infos.emit(tasks)
            self.clear_old()

    def slot_btn_deleteSelect(self):
        _indexes = self.list_view.selectionModel().selection().indexes()
        if not _indexes:
            return
        indexes = []
        for i in _indexes:  # 获取所选行号
            indexes.append(i.row())
        indexes = set(indexes)
        for i in sorted(indexes, reverse=True):
            self.selected.remove(self.model.item(i, 0).text())
            self.model.removeRow(i)
        self.set_size()

    def add_drop_files(self, files):
        for item in files:
            if item not in self.selected:
                self.selected.append(item)
            self.show_selected()

    def slot_btn_chooseDir(self):
        dir_choose = QFileDialog.getExistingDirectory(self, "选择文件夹", self.cwd)  # 起始路径
        if dir_choose == "":
            return
        if dir_choose not in self.selected:
            self.selected.append(dir_choose)
            self.cwd = os.path.dirname(dir_choose)
        self.show_selected()

    def slot_btn_chooseMultiFile(self):
        files, _ = QFileDialog.getOpenFileNames(self, "选择多文件", self.cwd, "All Files (*)")
        if len(files) == 0:
            return
        for _file in files:
            if _file not in self.selected:
                if os.path.getsize(_file) <= self.max_size * 1048576:
                    self.selected.append(_file)
                elif self.allow_big_file:
                    self.selected.append(_file)
        self.show_selected()

    def keyPressEvent(self, e):
        if e.key() == Qt.Key.Key_Delete:  # delete
            self.slot_btn_deleteSelect()



from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.api import LanZouCloud
from lanzou.debug import logger


class LoginLuncher(QThread):
    '''登录线程'''
    code = pyqtSignal(bool, str, int)
    update_cookie = pyqtSignal(object)
    update_username = pyqtSignal(object)

    def __init__(self, parent=None):
        super(LoginLuncher, self).__init__(parent)
        self._disk = None
        self.username = ""
        self.password = ""
        self.cookie = None

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, username, password, cookie=None):
        self.username = username
        self.password = password
        self.cookie = cookie
        self.start()

    def run(self):
        try:
            if self.cookie:
                res = self._disk.login_by_cookie(self.cookie)
                if res == LanZouCloud.SUCCESS:
                    if not self.username:
                        username = self._disk.get_user_name()
                        if isinstance(username, str):
                            self.update_username.emit(username)
                        logger.debug(f"login by Cookie: username={username}")
                    self.code.emit(True, "<font color='#00CC00'>通过<b>Cookie</b>登录<b>成功</b>！ ≧◉◡◉≦</font>", 5000)
                    return None
                logger.debug(f"login by Cookie err: res={res}")
            if (not self.username or not self.password) and not self.cookie:
                logger.debug("login err: No UserName、No cookie")
                self.code.emit(False, "<font color='red'>登录失败: 没有用户或密码</font>", 3000)
            else:
                res = self._disk.login(self.username, self.password)
                if res == LanZouCloud.SUCCESS:
                    self.code.emit(True, "<font color='#00CC00'>登录<b>成功</b>！ ≧◉◡◉≦</font>", 5000)
                    _cookie = self._disk.get_cookie()
                    self.update_cookie.emit(_cookie)
                else:
                    logger.debug(f"login err: res={res}")
                    self.code.emit(False, "<font color='red'>登录失败，可能是用户名或密码错误！</font>", 8000)
                    self.update_cookie.emit(None)
        except TimeoutError:
            self.code.emit(False, "<font color='red'>网络超时！</font>", 3000)
        except Exception as e:
            logger.error(f"LoginLuncher error: e={e}")


class LogoutWorker(QThread):
    '''登出'''
    succeeded = pyqtSignal()
    msg = pyqtSignal(str, int)

    def __init__(self, parent=None):
        super(LogoutWorker, self).__init__(parent)
        self._disk = None
        self.update_ui = True
        self._mutex = QMutex()
        self._is_work = False

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, update_ui=True):
        self.update_ui = update_ui
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            try:
                res = self._disk.logout()
                if res == LanZouCloud.SUCCESS:
                    if self.update_ui:
                        self.succeeded.emit()
                    self.msg.emit("已经退出登录！", 4000)
                else:
                    self.msg.emit("失败，请重试！", 5000)
            except TimeoutError:
                self.msg.emit("网络超时，请稍后重试！", 6000)
            except Exception as e:
                logger.error(f"LogoutWorker error: e={e}")
            self._is_work = False
            self._mutex.unlock()
        else:
            self.msg.emit("后台正在运行，请稍后重试！", 3100)



from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.debug import logger


class RemoveFilesWorker(QThread):
    '''删除文件(夹)线程'''
    msg = pyqtSignal(object, object)
    finished = pyqtSignal()

    def __init__(self, parent=None):
        super(RemoveFilesWorker, self).__init__(parent)
        self._disk = None
        self.infos = None
        self._mutex = QMutex()
        self._is_work = False

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, infos):
        self.infos = infos
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            if not self.infos:
                self._is_work = False
                self._mutex.unlock()
                return
            for i in self.infos:
                try:
                    self._disk.delete(i['fid'], i['is_file'])
                except TimeoutError:
                    self.msg.emit(f"删除 {i['name']} 因网络超时失败！", 3000)
                except Exception as e:
                    logger.error(f"RemoveFileWorker error: e={e}")
            self.finished.emit()
            self._is_work = False
            self._mutex.unlock()
        else:
            self.msg.emit("后台正在运行删除指令！", 3100)



from lanzou.gui.workers.desc import DescPwdFetcher
from lanzou.gui.workers.folders import GetAllFoldersWorker
from lanzou.gui.workers.login import LoginLuncher, LogoutWorker
from lanzou.gui.workers.manager import TaskManager
from lanzou.gui.workers.more import GetMoreInfoWorker
from lanzou.gui.workers.pwd import SetPwdWorker
from lanzou.gui.workers.recovery import GetRecListsWorker, RecManipulator
from lanzou.gui.workers.refresh import ListRefresher
from lanzou.gui.workers.rename import RenameMkdirWorker
from lanzou.gui.workers.rm import RemoveFilesWorker
from lanzou.gui.workers.share import GetSharedInfo
from lanzou.gui.workers.update import CheckUpdateWorker

__all__ = ['TaskManager', 'GetSharedInfo', 'LoginLuncher', 'DescPwdFetcher',
           'ListRefresher', 'GetRecListsWorker', 'RemoveFilesWorker',
           'GetMoreInfoWorker', 'GetAllFoldersWorker', 'RenameMkdirWorker',
           'SetPwdWorker', 'LogoutWorker', 'RecManipulator', 'CheckUpdateWorker']
